import {inject, singleton} from 'tsyringe'
import {models} from '@tonality/library-common'
import Uuid from '@tonality/library-uuid'
import {Controller, ControllerResponse, PaginatedResponse} from '../../utils/helpers'
import RingtoneService from './service'
import {DoubleDeletionError} from './response'

type RingtoneController = Controller<{
	get: ControllerResponse<models.Ringtone>,
	browse: PaginatedResponse<models.Ringtone>,
	search: ControllerResponse<models.Ringtone>,
	create: ControllerResponse<models.Ringtone>,
	update: ControllerResponse<models.Ringtone>,
	softDelete: ControllerResponse<models.Ringtone>,
	undoDelete: ControllerResponse<models.Ringtone>,
	hardDelete: undefined,
}>

export default RingtoneController

@singleton()
export class RingtoneControllerImpl implements RingtoneController {
	constructor(
		@inject('RingtoneService')
		private readonly ringtoneService: RingtoneService,
	) {}

	get = async (request: any, reply: any) => {
		try {
			const id = Uuid.parse(request.params['id'])
			const data = await this.ringtoneService.get(id)
			if (typeof (data.deletedAt as Date) !== 'undefined') {
				reply.raw.statusMessage = 'Ringtone Deleted Previously'
				reply.gone()
				return
			}
			if (!data) {
				reply.raw.statusMessage = 'Ringtone Not Found'
				reply.notFound()
				return
			}
			reply.raw.statusMessage = 'Single Ringtone Retrieved'
			return {
				data,
			}
		} catch (err) {
			reply.raw.statusMessage = 'Get Ringtone Error'
			reply.internalServerError(err.message)
		}
	}

	browse = async (request: any, reply: any) => {
		try {
			// TODO deserialize query
			const { 'skip': skipRaw, 'take': takeRaw } = request.query
			const skipNumber = Number(skipRaw)
			const takeNumber = Number(takeRaw)

			const skip = !isNaN(skipNumber) ? skipNumber : 0
			const take = !isNaN(takeNumber) ? takeNumber : 10

			reply.raw.statusMessage = 'Multiple Ringtones Retrieved'
			return {
				data: await this.ringtoneService.browse(skip, take),
				skip,
				take,
			}
		} catch (err) {
			reply.raw.statusMessage = 'Browse Ringtones Error'
			reply.internalServerError(err.message)
		}
	}

	search = async (request: any, reply: any) => {
		try {
			const { 'q': query } = request.query
			reply.raw.statusMessage = 'Search Results Retrieved'
			return {
				data: await this.ringtoneService.search(query),
			}
		} catch (err) {
			reply.raw.statusMessage = 'Search Error'
			reply.internalServerError(err.message)
		}
	}

	create = async (request: any, reply: any) => {
		try {
			const { composerUserId, ...etcBody } = request.body
			const data = await this.ringtoneService.create({
				...etcBody,
				composerUserId: Uuid.parse(composerUserId),
			})
			reply.raw.statusMessage = 'Ringtone Created'
			reply.raw.statusCode = 201
			return {
				data,
			}
		} catch (err) {
			reply.raw.statusMessage = 'Create Ringtone Error'
			reply.internalServerError(err.message)
		}
	}

	update = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const id = Uuid.parse(request.params['id'])
			const data = await this.ringtoneService.update({
				...request.body,
				id,
			})
			if (data.deletedAt) {
				reply.raw.statusMessage = 'Ringtone Deleted Previously'
				reply.gone()
				return
			}
			if (!data) {
				reply.raw.statusMessage = 'Ringtone Not Found'
				reply.notFound()
				return
			}
			reply.raw.statusMessage = 'Ringtone Updated'
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	softDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const id = Uuid.parse(request.params['id'])
			const data = await this.ringtoneService.softDelete(id)
			if (!data) {
				reply.raw.statusMessage = 'Ringtone Not Found'
				reply.notFound()
				return
			}
			reply.raw.statusMessage = 'Ringtone Soft-Deleted'
			return {
				data,
			}
		} catch (err) {
			reply.raw.statusMessage = 'Soft-Delete Ringtone Error'
			reply.internalServerError(err.message)
		}
	}

	undoDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const id = Uuid.parse(request.params['id'])
			const data = await this.ringtoneService.undoDelete(id)
			if (!data) {
				reply.raw.statusMessage = 'Ringtone Not Found'
				reply.notFound()
				return
			}
			reply.raw.statusMessage = 'Ringtone Restored'
			return {
				data,
			}
		} catch (err) {
			reply.raw.statusMessage = 'Restore Ringtone Error'
			reply.internalServerError(err.message)
		}
	}

	hardDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const id = Uuid.parse(request.params['id'])
			await this.ringtoneService.hardDelete(id)
			reply.status(204)
			reply.raw.statusMessage = 'Ringtone Hard-Deleted'
		} catch (err) {
			if (err instanceof DoubleDeletionError) {
				reply.raw.statusMessage = 'Ringtone Not Found'
				reply.notFound(err.message)
			}
			reply.raw.statusMessage = 'Delete Ringtone Error'
			reply.internalServerError(err.message)
		}
	}
}
