import {inject, singleton} from 'tsyringe'
import {models} from '@tonality/library-common'
import Uuid from '@tonality/library-uuid'
import {PrismaClient} from '@prisma/client'

export default interface RingtoneService {
	get(id: Uuid): Promise<models.Ringtone>
	browse(skip?: number, take?: number): Promise<models.Ringtone[]>
	search(q?: string): Promise<models.Ringtone>
	create(data: Partial<models.Ringtone>): Promise<models.Ringtone>
	update(data: Partial<models.Ringtone>): Promise<models.Ringtone>
	softDelete(id: Uuid): Promise<models.Ringtone>
	undoDelete(id: Uuid): Promise<models.Ringtone>
	hardDelete(id: Uuid): Promise<void>
}

@singleton()
export class RingtoneServiceImpl {
	constructor(
		@inject('PrismaClient')
		private readonly prismaClient: PrismaClient,
	) {}

	async get(id: Uuid): Promise<models.Ringtone> {
		const ringtone = await this.prismaClient.ringtone.findUnique({
			where: {
				id,
			}
		})

		if (!ringtone) {
			throw new Error('Ringtone not found!')
		}

		return ringtone
	}

	async browse(skip: number = 0, take: number = 10): Promise<models.Ringtone[]> {
		return this.prismaClient.ringtone.findMany({
			skip,
			take,
		})
	}

	async search(q: string = ''): Promise<models.Ringtone[]> {
		return this.prismaClient.ringtone.findMany({
			where: {
				name: {
					contains: q,
				},
			},
		})
	}

	async create(data: Partial<models.Ringtone>): Promise<models.Ringtone> {
		const { createdAt, updatedAt, deletedAt, composerUserId, name, ...safeData } = data
		return this.prismaClient.ringtone.create({
			data: {
				...safeData,
				id: Uuid.new(),
				composerUserId: composerUserId as Uuid,
				name: name as string,
			},
		})
	}

	async update(data: Partial<models.Ringtone>): Promise<models.Ringtone> {
		const { createdAt, updatedAt, deletedAt, ...safeData } = data
		return this.prismaClient.ringtone.update({
			where: {
				id: data.id as Uuid,
			},
			data: {
				...safeData,
				updatedAt: new Date(),
			},
		})
	}

	async softDelete(id: Uuid): Promise<models.Ringtone> {
		return this.prismaClient.ringtone.update({
			where: {
				id,
			},
			data: {
				deletedAt: new Date(),
			},
		})
	}

	async undoDelete(id: Uuid): Promise<models.Ringtone> {
		return this.prismaClient.ringtone.update({
			where: {
				id,
			},
			data: {
				deletedAt: null,
			},
		})
	}

	async hardDelete(id: Uuid): Promise<void> {
		await this.prismaClient.ringtone.delete({
			where: {
				id,
			},
		})
	}
}
